@@ -8,10 +8,12 @@ import os
 import shutil
 import sys
 import textwrap
+import warnings
 from argparse import Namespace
 from collections import defaultdict
 from itertools import chain
-from typing import Collection, Iterable, Mapping, Sequence, cast
+from pathlib import Path
+from typing import Collection, Iterable, Mapping, cast
 
 import tomlkit
 from resolvelib.reporters import BaseReporter
@@ -19,6 +21,7 @@ from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep, Resolv
 from tomlkit.items import Array
 
 from pdm import termui
+from pdm.cli.filters import GroupSelection
 from pdm.cli.hooks import HookManager
 from pdm.cli.utils import (
     check_project_file,
@@ -30,15 +33,14 @@ from pdm.cli.utils import (
     merge_dictionary,
     save_version_specifiers,
     set_env_in_reg,
-    translate_groups,
 )
+from pdm.environments import BareEnvironment, PythonLocalEnvironment
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
-from pdm.models.backends import BuildBackend
+from pdm.models.backends import DEFAULT_BACKEND, BuildBackend
 from pdm.models.caches import JSONFileCache
 from pdm.models.candidates import Candidate
-from pdm.models.environment import BareEnvironment
 from pdm.models.python import PythonInfo
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import get_specifier
@@ -54,6 +56,7 @@ def do_lock(
     requirements: list[Requirement] | None = None,
     dry_run: bool = False,
     refresh: bool = False,
+    groups: list[str] | None = None,
     hooks: HookManager | None = None,
 ) -> dict[str, Candidate]:
     """Performs the locking process and update lockfile."""
@@ -74,13 +77,13 @@ def do_lock(
             with project.core.ui.logging("lock"):
                 fetch_hashes(repo, mapping)
             lockfile = format_lockfile(project, mapping, dependencies)
-        project.write_lockfile(lockfile)
+        project.write_lockfile(lockfile, groups=groups)
         return mapping
     # TODO: multiple dependency definitions for the same package.
     provider = project.get_provider(strategy, tracked_names)
     if not requirements:
         requirements = [
-            r for deps in project.all_dependencies.values() for r in deps.values()
+            r for g, deps in project.all_dependencies.items() if groups is None or g in groups for r in deps.values()
         ]
     resolve_max_rounds = int(project.config["strategy.resolve_max_rounds"])
     ui = project.core.ui
@@ -117,21 +120,17 @@ def do_lock(
         else:
             data = format_lockfile(project, mapping, dependencies)
             ui.echo(f"{termui.Emoji.LOCK} Lock successful")
-            project.write_lockfile(data, write=not dry_run)
+            project.write_lockfile(data, write=not dry_run, groups=groups)
             hooks.try_emit("post_lock", resolution=mapping, dry_run=dry_run)
 
     return mapping
 
 
-def resolve_candidates_from_lockfile(
-    project: Project, requirements: Iterable[Requirement]
-) -> dict[str, Candidate]:
+def resolve_candidates_from_lockfile(project: Project, requirements: Iterable[Requirement]) -> dict[str, Candidate]:
     ui = project.core.ui
     resolve_max_rounds = int(project.config["strategy.resolve_max_rounds"])
     reqs = [
-        req
-        for req in requirements
-        if not req.marker or req.marker.evaluate(project.environment.marker_environment)
+        req for req in requirements if not req.marker or req.marker.evaluate(project.environment.marker_environment)
     ]
     with ui.logging("install-resolve"):
         with ui.open_spinner("Resolving packages from lockfile...") as spinner:
@@ -151,7 +150,7 @@ def resolve_candidates_from_lockfile(
 
 def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None:
     """Check if the lock file exists and is up to date. Return the update strategy."""
-    if not project.lockfile.exists:
+    if not project.lockfile.exists():
         if raise_not_exist:
             raise ProjectError("Lock file does not exist, nothing to install")
         project.core.ui.echo("Lock file does not exist", style="warning", err=True)
@@ -176,9 +175,7 @@ def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None
 def do_sync(
     project: Project,
     *,
-    groups: Collection[str] = (),
-    dev: bool = True,
-    default: bool = True,
+    selection: GroupSelection,
     dry_run: bool = False,
     clean: bool = False,
     requirements: list[Requirement] | None = None,
@@ -187,40 +184,41 @@ def do_sync(
     no_self: bool = False,
     reinstall: bool = False,
     only_keep: bool = False,
+    fail_fast: bool = False,
     hooks: HookManager | None = None,
 ) -> None:
     """Synchronize project"""
     hooks = hooks or HookManager(project)
     if requirements is None:
-        groups = translate_groups(project, default, dev, groups or ())
         requirements = []
-        for group in groups:
+        selection.validate()
+        for group in selection:
             requirements.extend(project.get_dependencies(group).values())
     candidates = resolve_candidates_from_lockfile(project, requirements)
     if tracked_names and dry_run:
-        candidates = {
-            name: c for name, c in candidates.items() if name in tracked_names
-        }
-    handler = project.core.synchronizer_class(
+        candidates = {name: c for name, c in candidates.items() if name in tracked_names}
+    synchronizer = project.core.synchronizer_class(
         candidates,
         project.environment,
-        clean,
-        dry_run,
+        clean=clean,
+        dry_run=dry_run,
         no_editable=no_editable,
-        install_self=not no_self and "default" in groups and bool(project.name),
+        install_self=not no_self and "default" in selection and bool(project.name),
         use_install_cache=project.config["install.cache"],
         reinstall=reinstall,
         only_keep=only_keep,
+        fail_fast=fail_fast,
     )
-    hooks.try_emit("pre_install", candidates=candidates, dry_run=dry_run)
-    handler.synchronize()
-    hooks.try_emit("post_install", candidates=candidates, dry_run=dry_run)
+    with project.core.ui.logging("install"):
+        hooks.try_emit("pre_install", candidates=candidates, dry_run=dry_run)
+        synchronizer.synchronize()
+        hooks.try_emit("post_install", candidates=candidates, dry_run=dry_run)
 
 
 def do_add(
     project: Project,
-    dev: bool = False,
-    group: str | None = None,
+    *,
+    selection: GroupSelection,
     sync: bool = True,
     save: str = "compatible",
     strategy: str = "reuse",
@@ -231,31 +229,25 @@ def do_add(
     no_self: bool = False,
     dry_run: bool = False,
     prerelease: bool = False,
+    fail_fast: bool = False,
     hooks: HookManager | None = None,
 ) -> None:
     """Add packages and install"""
     hooks = hooks or HookManager(project)
     check_project_file(project)
-    if not editables and not packages:
-        raise PdmUsageError("Must specify at least one package or editable package.")
     if editables and no_editable:
         raise PdmUsageError("Cannot use --no-editable with editable packages given.")
-    if not group:
-        group = "dev" if dev else "default"
+    group = selection.one()
     tracked_names: set[str] = set()
     requirements: dict[str, Requirement] = {}
-    if (
-        group == "default"
-        or not dev
-        and group not in project.pyproject.settings.get("dev-dependencies", {})
-    ):
+    lock_groups = [] if project.lockfile.empty() else project.lockfile.groups
+    if lock_groups is not None and group not in lock_groups:
+        project.core.ui.echo(f"Adding group [success]{group}[/] to lockfile", err=True, style="info")
+        lock_groups.append(group)
+    if group == "default" or not selection.dev and group not in project.pyproject.settings.get("dev-dependencies", {}):
         if editables:
-            raise PdmUsageError(
-                "Cannot add editables to the default or optional dependency group"
-            )
-    for r in [parse_requirement(line, True) for line in editables] + [
-        parse_requirement(line) for line in packages
-    ]:
+            raise PdmUsageError("Cannot add editables to the default or optional dependency group")
+    for r in [parse_requirement(line, True) for line in editables] + [parse_requirement(line) for line in packages]:
         if project.name and normalize_name(project.name) == r.key and not r.extras:
             project.core.ui.echo(
                 f"Package [req]{project.name}[/] is the project itself.",
@@ -264,47 +256,54 @@ def do_add(
             )
             continue
         if r.is_file_or_url:
-            r.relocate(project.backend)  # type: ignore
+            r.relocate(project.backend)  # type: ignore[attr-defined]
         key = r.identify()
         r.prerelease = prerelease
         tracked_names.add(key)
         requirements[key] = r
-    if not requirements:
-        return
-    project.core.ui.echo(
-        f"Adding packages to [primary]{group}[/] "
-        f"{'dev-' if dev else ''}dependencies: "
-        + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements.values())
-    )
+    if requirements:
+        project.core.ui.echo(
+            f"Adding packages to [primary]{group}[/] "
+            f"{'dev-' if selection.dev else ''}dependencies: "
+            + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements.values())
+        )
     all_dependencies = project.all_dependencies
     group_deps = all_dependencies.setdefault(group, {})
     if unconstrained:
+        if not requirements:
+            raise PdmUsageError("--unconstrained requires at least one package")
         for req in group_deps.values():
             req.specifier = get_specifier("")
     group_deps.update(requirements)
-    reqs = [r for deps in all_dependencies.values() for r in deps.values()]
+    reqs = [r for g, deps in all_dependencies.items() if lock_groups is None or g in lock_groups for r in deps.values()]
     with hooks.skipping("post_lock"):
         resolved = do_lock(
-            project, strategy, tracked_names, reqs, dry_run=dry_run, hooks=hooks
+            project,
+            strategy,
+            tracked_names,
+            reqs,
+            dry_run=True,
+            hooks=hooks,
+            groups=lock_groups,
         )
 
     # Update dependency specifiers and lockfile hash.
     deps_to_update = group_deps if unconstrained else requirements
     save_version_specifiers({group: deps_to_update}, resolved, save)
     if not dry_run:
-        project.add_dependencies(deps_to_update, group, dev)
-        project.write_lockfile(project.lockfile._data, False)
+        project.add_dependencies(deps_to_update, group, selection.dev or False)
+        project.write_lockfile(project.lockfile._data, False, groups=lock_groups)
         hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
     _populate_requirement_names(group_deps)
     if sync:
         do_sync(
             project,
-            groups=(group,),
-            default=False,
+            selection=GroupSelection(project, groups=[group], default=False),
             no_editable=no_editable and tracked_names,
             no_self=no_self,
             requirements=list(group_deps.values()),
             dry_run=dry_run,
+            fail_fast=fail_fast,
             hooks=hooks,
         )
 
@@ -320,9 +319,7 @@ def _populate_requirement_names(req_mapping: dict[str, Requirement]) -> None:
 def do_update(
     project: Project,
     *,
-    dev: bool | None = None,
-    groups: Sequence[str] = (),
-    default: bool = True,
+    selection: GroupSelection,
     strategy: str = "reuse",
     save: str = "compatible",
     unconstrained: bool = False,
@@ -333,49 +330,43 @@ def do_update(
     no_editable: bool = False,
     no_self: bool = False,
     prerelease: bool = False,
+    fail_fast: bool = False,
     hooks: HookManager | None = None,
 ) -> None:
     """Update specified packages or all packages"""
     hooks = hooks or HookManager(project)
     check_project_file(project)
-    if len(packages) > 0 and (top or len(groups) > 1 or not default):
-        raise PdmUsageError(
-            "packages argument can't be used together with multiple -G or "
-            "--no-default and --top."
-        )
+    if len(packages) > 0 and (top or selection.groups or not selection.default):
+        raise PdmUsageError("packages argument can't be used together with multiple -G or " "--no-default or --top.")
     all_dependencies = project.all_dependencies
     updated_deps: dict[str, dict[str, Requirement]] = defaultdict(dict)
-    install_dev = True if dev is None else dev
+    locked_groups = project.lockfile.groups
     if not packages:
         if prerelease:
             raise PdmUsageError("--prerelease must be used with packages given")
-        groups = translate_groups(project, default, install_dev, groups or ())
-        for group in groups:
+        selection.validate()
+        for group in selection:
             updated_deps[group] = all_dependencies[group]
     else:
-        group = groups[0] if groups else ("dev" if dev else "default")
+        group = selection.one()
+        if locked_groups and group not in locked_groups:
+            raise ProjectError(f"Requested group not in lockfile: {group}")
         dependencies = all_dependencies[group]
         for name in packages:
             matched_name = next(
-                (
-                    k
-                    for k in dependencies
-                    if normalize_name(strip_extras(k)[0]) == normalize_name(name)
-                ),
+                (k for k in dependencies if normalize_name(strip_extras(k)[0]) == normalize_name(name)),
                 None,
             )
             if not matched_name:
                 raise ProjectError(
                     f"[req]{name}[/] does not exist in [primary]{group}[/] "
-                    f"{'dev-' if dev else ''}dependencies."
+                    f"{'dev-' if selection.dev else ''}dependencies."
                 )
             dependencies[matched_name].prerelease = prerelease
             updated_deps[group][matched_name] = dependencies[matched_name]
         project.core.ui.echo(
             "Updating packages: {}.".format(
-                ", ".join(
-                    f"[req]{v}[/]" for v in chain.from_iterable(updated_deps.values())
-                )
+                ", ".join(f"[req]{v}[/]" for v in chain.from_iterable(updated_deps.values()))
             )
         )
     if unconstrained:
@@ -388,44 +379,43 @@ def do_update(
         strategy,
         chain.from_iterable(updated_deps.values()),
         reqs,
-        dry_run=dry_run,
+        dry_run=True,
         hooks=hooks,
+        groups=locked_groups,
     )
     for deps in updated_deps.values():
         _populate_requirement_names(deps)
-    if unconstrained and not dry_run:
+    if unconstrained:
         # Need to update version constraints
         save_version_specifiers(updated_deps, resolved, save)
         for group, deps in updated_deps.items():
-            project.add_dependencies(deps, group, dev or False)
-        project.write_lockfile(project.lockfile._data, False)
+            project.add_dependencies(deps, group, selection.dev or False)
+    if not dry_run:
+        project.write_lockfile(project.lockfile._data, False, groups=locked_groups)
     if sync or dry_run:
         do_sync(
             project,
-            groups=groups,
-            dev=install_dev,
-            default=default,
+            selection=selection,
             clean=False,
             dry_run=dry_run,
             requirements=[r for deps in updated_deps.values() for r in deps.values()],
-            tracked_names=list(chain.from_iterable(updated_deps.values()))
-            if top
-            else None,
+            tracked_names=list(chain.from_iterable(updated_deps.values())) if top else None,
             no_editable=no_editable,
             no_self=no_self,
+            fail_fast=fail_fast,
             hooks=hooks,
         )
 
 
 def do_remove(
     project: Project,
-    dev: bool = False,
-    group: str | None = None,
+    selection: GroupSelection,
     sync: bool = True,
     packages: Collection[str] = (),
     no_editable: bool = False,
     no_self: bool = False,
     dry_run: bool = False,
+    fail_fast: bool = False,
     hooks: HookManager | None = None,
 ) -> None:
     """Remove packages from working set and pyproject.toml"""
@@ -433,44 +423,39 @@ def do_remove(
     check_project_file(project)
     if not packages:
         raise PdmUsageError("Must specify at least one package to remove.")
-    if not group:
-        group = "dev" if dev else "default"
-    if group not in list(project.iter_groups()):
-        raise ProjectError(f"No-exist group {group}")
+    group = selection.one()
+    lock_groups = project.lockfile.groups
 
-    deps, _ = project.get_pyproject_dependencies(group, dev)
+    deps, _ = project.get_pyproject_dependencies(group, selection.dev or False)
     project.core.ui.echo(
         f"Removing packages from [primary]{group}[/] "
-        f"{'dev-' if dev else ''}dependencies: "
-        + ", ".join(f"[req]{name}[/]" for name in packages)
+        f"{'dev-' if selection.dev else ''}dependencies: " + ", ".join(f"[req]{name}[/]" for name in packages)
     )
     with cd(project.root):
         for name in packages:
             req = parse_requirement(name)
-            matched_indexes = sorted(
-                (i for i, r in enumerate(deps) if req.matches(r)), reverse=True
-            )
+            matched_indexes = sorted((i for i, r in enumerate(deps) if req.matches(r)), reverse=True)
             if not matched_indexes:
-                raise ProjectError(
-                    f"[req]{name}[/] does not exist in "
-                    f"[primary]{group}[/] dependencies."
-                )
+                raise ProjectError(f"[req]{name}[/] does not exist in [primary]{group}[/] dependencies.")
             for i in matched_indexes:
                 del deps[i]
     cast(Array, deps).multiline(True)
 
     if not dry_run:
         project.pyproject.write()
-    do_lock(project, "reuse", dry_run=dry_run, hooks=hooks)
+    if lock_groups and group not in lock_groups:
+        project.core.ui.echo(f"Group [success]{group}[/] isn't in lockfile, skipping lock.", style="warning", err=True)
+        return
+    do_lock(project, "reuse", dry_run=dry_run, hooks=hooks, groups=lock_groups)
     if sync:
         do_sync(
             project,
-            groups=(group,),
-            default=False,
+            selection=GroupSelection(project, default=False, groups=[group]),
             clean=True,
             no_editable=no_editable,
             no_self=no_self,
             dry_run=dry_run,
+            fail_fast=fail_fast,
             hooks=hooks,
         )
 
@@ -505,16 +490,12 @@ def do_build(
     with project.core.ui.logging("build"):
         if sdist:
             project.core.ui.echo("Building sdist...")
-            loc = SdistBuilder(project.root, project.environment).build(
-                dest, config_settings
-            )
+            loc = SdistBuilder(project.root, project.environment).build(dest, config_settings)
             project.core.ui.echo(f"Built sdist at {loc}")
             artifacts.append(loc)
         if wheel:
             project.core.ui.echo("Building wheel...")
-            loc = WheelBuilder(project.root, project.environment).build(
-                dest, config_settings
-            )
+            loc = WheelBuilder(project.root, project.environment).build(dest, config_settings)
             project.core.ui.echo(f"Built wheel at {loc}")
             artifacts.append(loc)
     hooks.try_emit("post_build", artifacts=artifacts, config_settings=config_settings)
@@ -547,19 +528,41 @@ def do_init(
     if build_backend is not None:
         data["build-system"] = build_backend.build_system()
     if python_requires and python_requires != "*":
-        data["project"]["requires-python"] = python_requires  # type: ignore
+        data["project"]["requires-python"] = python_requires
     if name and version:
         readme = next(project.root.glob("README*"), None)
         if readme is None:
             readme = project.root.joinpath("README.md")
-            readme.write_text(f"# {name}\n\n{description}\n")
-        data["project"]["readme"] = readme.name  # type: ignore
+            readme.write_text(f"# {name}\n\n{description}\n", encoding="utf-8")
+        data["project"]["readme"] = readme.name
     get_specifier(python_requires)
     project.pyproject._data.update(data)
     project.pyproject.write()
+    _write_gitignore(project.root.joinpath(".gitignore"))
     hooks.try_emit("post_init")
 
 
+def _write_gitignore(path: Path) -> None:
+    import requests
+
+    url = "https://raw.githubusercontent.com/github/gitignore/master/Python.gitignore"
+    if not path.exists():
+        try:
+            resp = requests.get(url)
+            resp.raise_for_status()
+        except requests.exceptions.RequestException:
+            content = "\n".join(["build/", "dist/", "*.egg-info/", "__pycache__/", "*.py[cod]"]) + "\n"
+        else:
+            content = resp.text
+        content += ".pdm-python\n"
+    else:
+        content = path.read_text(encoding="utf-8")
+        if ".pdm-python" in content:
+            return
+        content += ".pdm-python\n"
+    path.write_text(content, encoding="utf-8")
+
+
 def do_use(
     project: Project,
     python: str = "",
@@ -567,29 +570,32 @@ def do_use(
     ignore_remembered: bool = False,
     ignore_requires_python: bool = False,
     save: bool = True,
+    venv: str | None = None,
     hooks: HookManager | None = None,
 ) -> PythonInfo:
     """Use the specified python version and save in project config.
     The python can be a version string or interpreter path.
     """
-    hooks = hooks or HookManager(project)
-
-    if python:
-        python = python.strip()
+    from pdm.cli.commands.venv.utils import get_venv_python, get_venv_with_name
 
     def version_matcher(py_version: PythonInfo) -> bool:
         return py_version.valid and (
-            ignore_requires_python
-            or project.python_requires.contains(str(py_version.version), True)
+            ignore_requires_python or project.python_requires.contains(str(py_version.version), True)
         )
 
+    hooks = hooks or HookManager(project)
+
+    selected_python: PythonInfo | None = None
+    if venv:
+        venv_path = get_venv_with_name(project, venv)
+        selected_python = PythonInfo.from_path(get_venv_python(venv_path))
+
     if not project.cache_dir.exists():
         project.cache_dir.mkdir(parents=True)
-    use_cache: JSONFileCache[str, str] = JSONFileCache(
-        project.cache_dir / "use_cache.json"
-    )
-    selected_python: PythonInfo | None = None
-    if python and not ignore_remembered:
+    use_cache: JSONFileCache[str, str] = JSONFileCache(project.cache_dir / "use_cache.json")
+    if python:
+        python = python.strip()
+    if selected_python is None and python and not ignore_remembered:
         if python in use_cache:
             path = use_cache.get(python)
             cached_python = PythonInfo.from_path(path)
@@ -611,27 +617,21 @@ def do_use(
         found_interpreters = list(dict.fromkeys(project.find_interpreters(python)))
         matching_interpreters = list(filter(version_matcher, found_interpreters))
         if not found_interpreters:
-            raise NoPythonVersion(
-                f"No Python interpreter matching [success]{python}[/] is found."
-            )
+            raise NoPythonVersion(f"No Python interpreter matching [success]{python}[/] is found.")
         if not matching_interpreters:
             project.core.ui.echo("Interpreters found but not matching:", err=True)
             for py in found_interpreters:
                 info = py.identifier if py.valid else "Invalid"
                 project.core.ui.echo(f"  - {py.path} ({info})", err=True)
             raise NoPythonVersion(
-                "No python is found meeting the requirement "
-                f"[success]python {str(project.python_requires)}[/]"
+                f"No python is found meeting the requirement [success]python {str(project.python_requires)}[/]"
             )
         if first or len(matching_interpreters) == 1:
             selected_python = matching_interpreters[0]
         else:
             project.core.ui.echo("Please enter the Python interpreter to use")
             for i, py_version in enumerate(matching_interpreters):
-                project.core.ui.echo(
-                    f"{i}. [success]{str(py_version.path)}[/] "
-                    f"({py_version.identifier})"
-                )
+                project.core.ui.echo(f"{i}. [success]{str(py_version.path)}[/] ({py_version.identifier})")
             selection = termui.ask(
                 "Please select",
                 default="0",
@@ -645,21 +645,22 @@ def do_use(
 
     if not save:
         return selected_python
-    old_python = (
-        PythonInfo.from_path(project.config["python.path"])
-        if "python.path" in project.config
-        else None
-    )
+    saved_python = project._saved_python
+    old_python = PythonInfo.from_path(saved_python) if saved_python else None
     project.core.ui.echo(
-        "Using Python interpreter: "
-        f"[success]{str(selected_python.path)}[/] "
-        f"({selected_python.identifier})"
+        f"Using Python interpreter: [success]{str(selected_python.path)}[/] ({selected_python.identifier})"
     )
     project.python = selected_python
+    if project.environment.is_local:
+        project.core.ui.echo(
+            "Using __pypackages__ because non-venv Python is used.",
+            style="primary",
+            err=True,
+        )
     if (
         old_python
         and old_python.executable != selected_python.executable
-        and not project.environment.is_global
+        and isinstance(project.environment, PythonLocalEnvironment)
     ):
         project.core.ui.echo("Updating executable scripts...", style="primary")
         project.environment.update_shebangs(selected_python.executable.as_posix())
@@ -686,8 +687,7 @@ def do_import(
                 break
         else:
             raise PdmUsageError(
-                "Can't derive the file format automatically, "
-                "please specify it via '-f/--format' option."
+                "Can't derive the file format automatically, please specify it via '-f/--format' option."
             )
     else:
         key = format
@@ -696,30 +696,21 @@ def do_import(
     project_data, settings = FORMATS[key].convert(project, filename, options)
     pyproject = project.pyproject._data
 
-    if "tool" not in pyproject or "pdm" not in pyproject["tool"]:  # type: ignore
+    if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
         pyproject.setdefault("tool", {})["pdm"] = tomlkit.table()
-    if "build" in pyproject["tool"]["pdm"] and isinstance(
-        pyproject["tool"]["pdm"]["build"], str
-    ):
+    if "build" in pyproject["tool"]["pdm"] and isinstance(pyproject["tool"]["pdm"]["build"], str):
         pyproject["tool"]["pdm"]["build"] = {
             "setup-script": pyproject["tool"]["pdm"]["build"],
             "run-setuptools": True,
         }
     if "project" not in pyproject:
-        pyproject.add("project", tomlkit.table())  # type: ignore
-        pyproject["project"].add(  # type: ignore
-            tomlkit.comment("PEP 621 project metadata")
-        )
-        pyproject["project"].add(  # type: ignore
-            tomlkit.comment("See https://www.python.org/dev/peps/pep-0621/")
-        )
+        pyproject.add("project", tomlkit.table())
+        pyproject["project"].add(tomlkit.comment("PEP 621 project metadata"))
+        pyproject["project"].add(tomlkit.comment("See https://www.python.org/dev/peps/pep-0621/"))
 
-    merge_dictionary(pyproject["project"], project_data)  # type: ignore
-    merge_dictionary(pyproject["tool"]["pdm"], settings)  # type: ignore
-    pyproject["build-system"] = {
-        "requires": ["pdm-pep517>=1.0.0"],
-        "build-backend": "pdm.pep517.api",
-    }
+    merge_dictionary(pyproject["project"], project_data)
+    merge_dictionary(pyproject["tool"]["pdm"], settings)
+    pyproject["build-system"] = DEFAULT_BACKEND.build_system()
 
     if "requires-python" not in pyproject["project"]:
         python_version = f"{project.python.major}.{project.python.minor}"
@@ -736,14 +727,10 @@ def ask_for_import(project: Project) -> None:
     importable_files = list(find_importable_files(project))
     if not importable_files:
         return
-    project.core.ui.echo(
-        "Found following files from other formats that you may import:", style="primary"
-    )
+    project.core.ui.echo("Found following files from other formats that you may import:", style="primary")
     for i, (key, filepath) in enumerate(importable_files):
         project.core.ui.echo(f"{i}. [success]{filepath.as_posix()}[/] ({key})")
-    project.core.ui.echo(
-        f"{len(importable_files)}. [warning]don't do anything, I will import later.[/]"
-    )
+    project.core.ui.echo(f"{len(importable_files)}. [warning]don't do anything, I will import later.[/]")
     choice = termui.ask(
         "Please select",
         prompt_type=int,
@@ -773,8 +760,7 @@ def print_pep582_command(project: Project, shell: str = "AUTO") -> None:
                 err=True,
             )
         ui.echo(
-            "The environment variable has been saved, "
-            "please restart the session to take effect.",
+            "The environment variable has been saved, please restart the session to take effect.",
             style="success",
         )
         return
@@ -793,15 +779,7 @@ def print_pep582_command(project: Project, shell: str = "AUTO") -> None:
             """
         ).strip()
     elif shell == "fish":
-        result = textwrap.dedent(
-            f"""
-            if test -n "$PYTHONPATH"
-                set -x PYTHONPATH '{lib_path}' $PYTHONPATH
-            else
-                set -x PYTHONPATH '{lib_path}'
-            end
-            """
-        ).strip()
+        result = f"set -x PYTHONPATH '{lib_path}' $PYTHONPATH"
     elif shell in ("tcsh", "csh"):
         result = textwrap.dedent(
             f"""
@@ -817,16 +795,11 @@ def print_pep582_command(project: Project, shell: str = "AUTO") -> None:
             """
         ).strip()
     else:
-        raise PdmUsageError(
-            f"Unsupported shell: {shell}, please specify another shell "
-            "via `--pep582 <SHELL>`"
-        )
+        raise PdmUsageError(f"Unsupported shell: {shell}, please specify another shell via `--pep582 <SHELL>`")
     ui.echo(result)
 
 
-def get_latest_pdm_version_from_pypi(
-    project: Project, prereleases: bool = False
-) -> str | None:
+def get_latest_pdm_version_from_pypi(project: Project, prereleases: bool = False) -> str | None:
     """Get the latest version of PDM from PyPI."""
     environment = BareEnvironment(project)
     with environment.get_finder([project.default_source]) as finder:
@@ -842,12 +815,13 @@ def get_latest_version(project: Project) -> str | None:
     with contextlib.suppress(OSError):
         state = json.loads(cache_file.read_text())
     current_time = datetime.datetime.utcnow().timestamp()
-    if (
-        state.get("last-check")
-        and current_time - state["last-check"] < 60 * 60 * 24 * 7
-    ):
+    if state.get("last-check") and current_time - state["last-check"] < 60 * 60 * 24 * 7:
         return cast(str, state["latest-version"])
-    latest_version = get_latest_pdm_version_from_pypi(project)
+    try:
+        latest_version = get_latest_pdm_version_from_pypi(project)
+    except Exception as e:
+        warnings.warn(f"Failed to get latest version: {e}", RuntimeWarning, stacklevel=1)
+        latest_version = None
     if latest_version is None:
         return None
     state.update({"latest-version": latest_version, "last-check": current_time})
@@ -858,15 +832,13 @@ def get_latest_version(project: Project) -> str | None:
 
 def check_update(project: Project) -> None:
     """Check if there is a new version of PDM available"""
-    from packaging.version import parse as parse_version
+    from packaging.version import Version
 
     this_version = project.core.version
     latest_version = get_latest_version(project)
-    if latest_version is None or parse_version(this_version) >= parse_version(
-        latest_version
-    ):
+    if latest_version is None or Version(this_version) >= Version(latest_version):
         return
-    install_command = "pdm self update"
+    install_command = "pdm self update" + (" --pre" if Version(latest_version).is_prerelease else "")
     disable_command = "pdm config check_update false"
 
     message = [
