@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import hashlib
 import os
 import re
@@ -15,12 +16,13 @@ from tomlkit.items import Array
 from unearth import Link
 
 from pdm import termui
+from pdm._types import RepositoryConfig
 from pdm.compat import cached_property
+from pdm.environments import BaseEnvironment, PythonEnvironment, PythonLocalEnvironment
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.models.backends import BuildBackend, get_backend_by_spec
 from pdm.models.caches import CandidateInfoCache, HashCache, WheelCache
 from pdm.models.candidates import Candidate, make_candidate
-from pdm.models.environment import Environment, GlobalEnvironment
 from pdm.models.python import PythonInfo
 from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
@@ -34,7 +36,6 @@ from pdm.utils import (
     expand_env_vars_in_auth,
     find_project_root,
     find_python_in_path,
-    get_venv_like_prefix,
     normalize_name,
     path_to_url,
 )
@@ -42,7 +43,7 @@ from pdm.utils import (
 if TYPE_CHECKING:
     from resolvelib.reporters import BaseReporter
 
-    from pdm._types import Source, Spinner
+    from pdm._types import Spinner
     from pdm.core import Core
     from pdm.resolver.providers import BaseProvider
 
@@ -72,7 +73,7 @@ class Project:
         global_config: str | Path | None = None,
     ) -> None:
         self._lockfile: Lockfile | None = None
-        self._environment: Environment | None = None
+        self._environment: BaseEnvironment | None = None
         self._python: PythonInfo | None = None
         self.core = core
 
@@ -87,11 +88,7 @@ class Project:
                 if not is_global
                 else global_project
             )
-        if (
-            not is_global
-            and root_path is None
-            and self.global_config["global_project.fallback"]
-        ):
+        if not is_global and root_path is None and self.global_config["global_project.fallback"]:
             root_path = global_project
             is_global = True
             if self.global_config["global_project.fallback_verbose"]:
@@ -115,9 +112,7 @@ class Project:
     @property
     def lockfile(self) -> Lockfile:
         if self._lockfile is None:
-            self._lockfile = Lockfile(
-                self.root / self.LOCKFILE_FILENAME, ui=self.core.ui
-            )
+            self._lockfile = Lockfile(self.root / self.LOCKFILE_FILENAME, ui=self.core.ui)
         return self._lockfile
 
     def set_lockfile(self, path: str | Path) -> None:
@@ -132,12 +127,17 @@ class Project:
 
     @property
     def scripts(self) -> dict[str, str | dict[str, str]]:
-        return self.pyproject.settings.get("scripts", {})  # type: ignore
+        return self.pyproject.settings.get("scripts", {})
 
     @cached_property
     def project_config(self) -> Config:
         """Read-and-writable configuration dict for project settings"""
-        return Config(self.root / ".pdm.toml")
+        config = Config(self.root / "pdm.toml")
+        # TODO: for backward compatibility, remove this in the future
+        if self.root.joinpath(".pdm.toml").exists():
+            legacy_config = Config(self.root / ".pdm.toml").self_data
+            config.update((k, v) for k, v in legacy_config.items() if k != "python.path")
+        return config
 
     @property
     def name(self) -> str | None:
@@ -157,7 +157,31 @@ class Project:
     @python.setter
     def python(self, value: PythonInfo) -> None:
         self._python = value
-        self.project_config["python.path"] = value.path
+        self._saved_python = value.path.as_posix()
+
+    @property
+    def _saved_python(self) -> str | None:
+        if os.getenv("PDM_PYTHON"):
+            return os.getenv("PDM_PYTHON")
+        with contextlib.suppress(FileNotFoundError):
+            return self.root.joinpath(".pdm-python").read_text("utf-8").strip()
+        with contextlib.suppress(FileNotFoundError):
+            # TODO: remove this in the future
+            with self.root.joinpath(".pdm.toml").open("rb") as fp:
+                data = tomlkit.load(fp)
+                if data.get("python", {}).get("path"):
+                    return data["python"]["path"]
+        return None
+
+    @_saved_python.setter
+    def _saved_python(self, value: str | None) -> None:
+        self.root.mkdir(parents=True, exist_ok=True)
+        python_file = self.root.joinpath(".pdm-python")
+        if value is None:
+            with contextlib.suppress(FileNotFoundError):
+                python_file.unlink()
+            return
+        python_file.write_text(value, "utf-8")
 
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
@@ -168,25 +192,30 @@ class Project:
 
         def note(message: str) -> None:
             if not self.is_global:
-                self.core.ui.echo(message, style="warning", err=True)
+                self.core.ui.echo(message, style="info", err=True)
 
         config = self.config
-        if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
-            saved_path = config["python.path"]
+        saved_path = self._saved_python
+        if saved_path and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
             python = PythonInfo.from_path(saved_path)
             if match_version(python):
                 return python
-            self.project_config.pop("python.path", None)
+            else:
+                note(
+                    "The saved Python interpreter doesn't match the project's requirement. "
+                    "Trying to find another one."
+                )
+            self._saved_python = None  # Clear the saved path if it doesn't match
 
-        if config.get("python.use_venv") and not self.is_global:
+        if config.get("python.use_venv") and not self.is_global and not os.getenv("PDM_IGNORE_ACTIVE_VENV"):
             # Resolve virtual environments from env-vars
             venv_in_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
             if venv_in_env:
                 python = PythonInfo.from_path(get_venv_python(Path(venv_in_env)))
                 if match_version(python):
                     note(
-                        f"Inside an active virtualenv [success]{venv_in_env}[/], "
-                        "reusing it."
+                        f"Inside an active virtualenv [success]{venv_in_env}[/], reusing it.\n"
+                        "Set env var [success]PDM_IGNORE_ACTIVE_VENV[/] to ignore it."
                     )
                     return python
             # otherwise, get a venv associated with the project
@@ -198,10 +227,7 @@ class Project:
                     return python
 
             if not self.root.joinpath("__pypackages__").exists():
-                note(
-                    "python.use_venv is on, creating a virtualenv for this project"
-                    "..."
-                )
+                note("python.use_venv is on, creating a virtualenv for this project...")
                 venv = self._create_virtualenv()
                 self.python = PythonInfo.from_path(get_venv_python(venv))
                 return self.python
@@ -209,33 +235,26 @@ class Project:
         for py_version in self.find_interpreters():
             if match_version(py_version):
                 if config.get("python.use_venv"):
-                    note(
-                        "[success]__pypackages__[/] is detected, using the PEP 582 mode"
-                    )
+                    note("[success]__pypackages__[/] is detected, using the PEP 582 mode")
                 self.python = py_version
                 return py_version
 
-        raise NoPythonVersion(
-            "No Python that satisfies {} is found on the system.".format(
-                self.python_requires
-            )
-        )
+        raise NoPythonVersion(f"No Python that satisfies {self.python_requires} is found on the system.")
 
-    def get_environment(self) -> Environment:
+    def get_environment(self) -> BaseEnvironment:
         """Get the environment selected by this project"""
 
         if self.is_global:
-            env = GlobalEnvironment(self)
+            env = PythonEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
             env.python_requires = PySpecSet(f"=={self.python.version}")
             return env
 
         return (
-            GlobalEnvironment(self)
-            if self.config["python.use_venv"]
-            and get_venv_like_prefix(self.python.executable) is not None
-            else Environment(self)
+            PythonEnvironment(self)
+            if self.config["python.use_venv"] and self.python.is_venv
+            else PythonLocalEnvironment(self)
         )
 
     def _create_virtualenv(self) -> Path:
@@ -247,20 +266,19 @@ class Project:
             force=True,
             in_project=self.config["venv.in_project"],
             prompt=self.config["venv.prompt"],
+            with_pip=self.config["venv.with_pip"],
         )
-        self.core.ui.echo(
-            f"Virtualenv is created successfully at [success]{path}[/]", err=True
-        )
+        self.core.ui.echo(f"Virtualenv is created successfully at [success]{path}[/]", err=True)
         return path
 
     @property
-    def environment(self) -> Environment:
+    def environment(self) -> BaseEnvironment:
         if not self._environment:
             self._environment = self.get_environment()
         return self._environment
 
     @environment.setter
-    def environment(self, value: Environment) -> None:
+    def environment(self, value: BaseEnvironment) -> None:
         self._environment = value
 
     @property
@@ -347,31 +365,41 @@ class Project:
         return self.pyproject.settings.get("allow_prereleases")
 
     @property
-    def default_source(self) -> Source:
+    def default_source(self) -> RepositoryConfig:
         """Get the default source from the pypi setting"""
-        return cast(
-            "Source",
-            {
-                "url": self.config["pypi.url"],
-                "verify_ssl": self.config["pypi.verify_ssl"],
-                "name": "pypi",
-                "username": self.config.get("pypi.username"),
-                "password": self.config.get("pypi.password"),
-            },
+        return RepositoryConfig(
+            name="pypi",
+            url=self.config["pypi.url"],
+            verify_ssl=self.config["pypi.verify_ssl"],
+            username=self.config.get("pypi.username"),
+            password=self.config.get("pypi.password"),
         )
 
     @property
-    def sources(self) -> list[Source]:
-        sources = list(self.pyproject.settings.get("source", []))
-        if not self.global_config["pypi.ignore_stored_index"]:
-            if all(source.get("name") != "pypi" for source in sources):
-                sources.insert(0, self.default_source)
-            sources.extend(self.global_config.iter_sources())
-        expanded_sources = [
-            cast("Source", {**source, "url": expand_env_vars_in_auth(source["url"])})
-            for source in sources
-        ]
-        return expanded_sources
+    def sources(self) -> list[RepositoryConfig]:
+        result: dict[str, RepositoryConfig] = {}
+        for source in self.pyproject.settings.get("source", []):
+            result[source["name"]] = RepositoryConfig(**source)
+
+        def merge_sources(other_sources: Iterable[tuple[str, RepositoryConfig]]) -> None:
+            for name, source in other_sources:
+                source.name = name
+                if name in result:
+                    result[name].passive_update(source)
+                else:
+                    result[name] = source
+
+        if not self.config.get("pypi.ignore_stored_index", False):
+            if "pypi" not in result:  # put pypi source at the beginning
+                result = {"pypi": self.default_source, **result}
+            else:
+                result["pypi"].passive_update(self.default_source)
+            merge_sources(self.project_config.iter_sources())
+            merge_sources(self.global_config.iter_sources())
+        for source in result.values():
+            assert source.url, "Source URL must not be empty"
+            source.url = expand_env_vars_in_auth(source.url)
+        return list(result.values())
 
     def get_repository(
         self, cls: type[BaseRepository] | None = None, ignore_compatibility: bool = True
@@ -414,9 +442,7 @@ class Project:
 
         repository = self.get_repository(ignore_compatibility=ignore_compatibility)
         allow_prereleases = self.allow_prereleases
-        overrides = {
-            normalize_name(k): v for k, v in self.pyproject.resolution_overrides.items()
-        }
+        overrides = {normalize_name(k): v for k, v in self.pyproject.resolution_overrides.items()}
         locked_repository: LockedRepository | None = None
         if strategy != "all" or for_install:
             try:
@@ -434,9 +460,7 @@ class Project:
             return BaseProvider(repository, allow_prereleases, overrides)
         if for_install:
             return BaseProvider(locked_repository, allow_prereleases, overrides)
-        provider_class = (
-            ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
-        )
+        provider_class = ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
         tracked_names = [strip_extras(name)[0] for name in tracked_names or ()]
         return provider_class(
             locked_repository.all_candidates,
@@ -466,18 +490,22 @@ class Project:
 
         return SpinnerReporter(spinner, requirements)
 
-    def get_lock_metadata(self) -> dict[str, Any]:
+    def get_lock_metadata(self, groups: Iterable[str] | None = None) -> dict[str, Any]:
         content_hash = tomlkit.string("sha256:" + self.pyproject.content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
+        if groups is None:
+            groups = self.iter_groups()
         return {
             "lock_version": self.lockfile.spec_version,
+            "groups": sorted(groups, key=lambda x: (x != "default", x)),
             "content_hash": content_hash,
         }
 
     def write_lockfile(
-        self, toml_data: dict, show_message: bool = True, write: bool = True
+        self, toml_data: dict, show_message: bool = True, write: bool = True, groups: Iterable[str] | None = None
     ) -> None:
-        toml_data["metadata"].update(self.get_lock_metadata())
+        """Write the lock file to disk."""
+        toml_data["metadata"].update(self.get_lock_metadata(groups))
         self.lockfile.set_data(toml_data)
 
         if write:
@@ -503,7 +531,7 @@ class Project:
         """Within the same major version, the higher lockfile generator can work with
         lower lockfile but not vice versa.
         """
-        if not self.lockfile.exists:
+        if not self.lockfile.exists():
             return True
         lockfile_version = str(self.lockfile.file_version)
         if not lockfile_version:
@@ -513,9 +541,7 @@ class Project:
         accepted = get_specifier(f"~={lockfile_version},>={lockfile_version}")
         return accepted.contains(self.lockfile.spec_version)
 
-    def get_pyproject_dependencies(
-        self, group: str, dev: bool = False
-    ) -> tuple[list[str], bool]:
+    def get_pyproject_dependencies(self, group: str, dev: bool = False) -> tuple[list[str], bool]:
         """Get the dependencies array in the pyproject.toml
         Return a tuple of two elements, the first is the dependencies array,
         and the second tells whether it is a dev-dependencies group.
@@ -563,12 +589,10 @@ class Project:
         self.pyproject.write(show_message)
 
     def init_global_project(self) -> None:
-        if not self.is_global or not self.pyproject.empty:
+        if not self.is_global or not self.pyproject.empty():
             return
         self.root.mkdir(parents=True, exist_ok=True)
-        self.pyproject.set_data(
-            {"project": {"dependencies": ["pip", "setuptools", "wheel"]}}
-        )
+        self.pyproject.set_data({"project": {"dependencies": ["pip", "setuptools", "wheel"]}})
         self.pyproject.write()
 
     @property
@@ -592,9 +616,7 @@ class Project:
         return WheelCache(self.cache("wheels"))
 
     def make_candidate_info_cache(self) -> CandidateInfoCache:
-        python_hash = hashlib.sha1(
-            str(self.environment.python_requires).encode()
-        ).hexdigest()
+        python_hash = hashlib.sha1(str(self.environment.python_requires).encode()).hexdigest()
         file_name = f"package_meta_{python_hash}.json"
         return CandidateInfoCache(self.cache("metadata") / file_name)
 
@@ -651,21 +673,16 @@ class Project:
 
         finder = Finder(resolve_symlinks=True)
         if self.config["python.use_venv"]:
-            finder._providers.insert(0, VenvProvider(self))
+            finder.add_provider(VenvProvider(self), 0)
         return finder
 
     # compatibility, shouldn't be used directly
     @property
     def meta(self) -> dict[str, Any]:
-        deprecation_warning(
-            "project.meta is deprecated, use project.pyproject.metadata instead"
-        )
+        deprecation_warning("project.meta is deprecated, use project.pyproject.metadata instead")
         return self.pyproject.metadata
 
     @property
     def tool_settings(self) -> dict[str, Any]:
-        deprecation_warning(
-            "project.tool_settings is deprecated, "
-            "use project.pyproject.settings instead"
-        )
+        deprecation_warning("project.tool_settings is deprecated, use project.pyproject.settings instead")
         return self.pyproject.settings
