@@ -72,6 +72,10 @@ def build_env_wheels() -> Iterable[Path]:
             "setuptools-65.4.1-py3-none-any.whl",
             "wheel-0.37.1-py2.py3-none-any.whl",
             "flit_core-3.6.0-py3-none-any.whl",
+            "pdm_backend-2.0.2-py3-none-any.whl",
+            "importlib_metadata-4.8.3-py3-none-any.whl",
+            "zipp-3.7.0-py3-none-any.whl",
+            "typing_extensions-4.4.0-py3-none-any.whl",
         )
     ]
 
@@ -99,6 +103,7 @@ def fixture_project(project_no_init):
         source = FIXTURES / "projects" / project_name
         copytree(source, project_no_init.root)
         project_no_init.pyproject.reload()
+        project_no_init.environment = None
         return project_no_init
 
     return func
@@ -116,8 +121,8 @@ def is_editable(request):
 
 
 @pytest.fixture(params=[False, True])
-def is_dev(request):
-    return request.param
+def dev_option(request) -> Iterable[str]:
+    return ("--dev",) if request.param else ()
 
 
 @pytest.fixture
