@@ -3,10 +3,10 @@ import pytest
 
 from pdm.utils import cd
 
-PYTHON_VERSIONS = ["3.6", "3.7", "3.8", "3.9", "3.10", "3.11"]
+PYTHON_VERSIONS = ["3.7", "3.8", "3.9", "3.10", "3.11"]
 PYPROJECT = {
-    "project": {"name": "test-project", "version": "0.1.0", "requires-python": ">=3.6"},
-    "build-system": {"requires": ["pdm-pep517"], "build-backend": "pdm.pep517.api"},
+    "project": {"name": "test-project", "version": "0.1.0", "requires-python": ">=3.7"},
+    "build-system": {"requires": ["pdm-backend"], "build-backend": "pdm.backend"},
 }
 
 
@@ -37,9 +37,7 @@ def test_basic_integration(python_version, core, tmp_path, invoke):
         invoke(["build", "-v"], obj=project, strict=True)
     invoke(["remove", "-v", "django"], obj=project, strict=True)
     result = invoke(["list"], obj=project, strict=True)
-    assert not any(
-        line.strip().lower().startswith("django") for line in result.output.splitlines()
-    )
+    assert not any(line.strip().lower().startswith("django") for line in result.output.splitlines())
 
 
 def test_actual_list_freeze(project, local_finder, invoke):
